/* Primal-Dual Vertex / Facet Enumeration
   Ambros Marzetta

 * $Source: /home/schilte/marzetta/work/pd/RCS/pd.c,v $
 * $Revision: 1.7 $
 * $Date: 1997/11/20 09:51:51 $
 * $Log: pd.c,v $
 * Revision 1.7  1997/11/20  09:51:51  marzetta
 * Added volume computation.
 *
 * Revision 1.6  1997/11/18  14:54:12  marzetta
 * Better error messages and warnings; fixed bug with incidence counting;
 * v0dict is now normalized after the translation.
 *
 * Revision 1.5  1997/10/20  17:40:36  marzetta
 * Added V-representation and H-representation.
 * A V-representation is now translated so that it contains the origin. To
 * avoid the translation, you can include the option "notranslation".
 *
 * Revision 1.4  1997/10/06  13:30:32  marzetta
 * New preprocessing (nd^3)
 *
 * Revision 1.3  1997/10/01  16:54:47  marzetta
 * Fixed bug with count[] on Sun
 *
 * Revision 1.2  1997/08/30  15:53:24  marzetta
 * Fixed bug in DeleteVertex3
 *
 * Revision 1.1  1997/04/18  15:01:16  marzetta
 * Initial revision
 *

 General information: http://wwwjn.inf.ethz.ch/ambros/pd.html
 Manual:              http://wwwjn.inf.ethz.ch/ambros/pd_man.html

 
 This program either reads a polytope in vertex representation from
 stdin and writes the halfspace representation to stdout.

 Precondition: the polytope must be full-dimensional.

 Or it reads a polytope in halfspace representation from stdin and
 writes the vertex representation to stdout.

 Precondition: The origin must be in the interior of the polyhedron
 (in the interior, not on the border!), and the polyhedron must have at
 least one vertex.

 
 For a description of the algorithm, see Primal-Dual Methods for Vertex
 and Facet Enumeration. David Bremner, Komei Fukuda and Ambros Marzetta.
 13th ACM Symposium on Computational Geometry 1997.

 The functions PointInDict() and DeleteVertex() are somewhat slower than
 the paper describes.
 */

#define TITLE "*Primal-Dual Vertex / Facet Enumeration "
#ifdef NDEBUG
#define VERSION "bit $Revision: 1.7 $"
#else /* NDEBUG */
#define VERSION "bit $Revision: 1.7 $ DEBUG"
#endif /* NDEBUG */
#define AUTHOR "\n*Copyright 1997 Ambros Marzetta marzetta@inf.ethz.ch"

#ifdef INFO
#define INF 1
#else
#define INF 0
#endif

#ifdef TRACE
#define TR 1
#else
#define TR 0
#endif


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "prs_mp.h"
#include <sys/time.h>		/* all Unix systems */
#include <sys/resource.h>	/* struct rusage */
/* int getrusage(int who, struct rusage *rusage); */

/*
   this is in decimal digits, you pay in memory if you increase this,
   unless you override by a line with 
   digits n
   before the begin line of your file.
*/
#define DEFAULT_DIGITS 180L


typedef char boolean;

typedef struct pd_Row {
  /* Row of dictionary */
  struct pd_Row *nextrow;	/* pointer to next row in list */
  long b;			/* number of variable */
				/* -m-1   = additional row for vertex enum */
				/* -m..-1 = hyperplane */
				/* 0      = objective function */
				/* 1..d   = coordinate */
  long minratio;		/* for ratio test */
  long data[1];			/* variable size: (digits+1) * (d+1) */
} pd_Row, *pd_RowP;

typedef struct pd_ColDesc {
  long c;			/* number of variable [-m..d] */
  long loc;			/* column in dictionary */
} pd_ColDesc;

typedef struct pd_Dict {
  /* Dictionary with variable number of rows */
  long m;			/* number of rows in dictionary (excl obj) */
  long detsign;
  pd_RowP firstrow, lastrow;	/* first and last element of row list */
  pd_RowP objrow;		/* objective function, in above list */
  mp det;
  pd_ColDesc cd[1];		/* variable size: d elements*/
} pd_Dict, *pd_DictP;

typedef struct pd_Global {	/* read-only for the search algorithm    */
  long d;			/* number of cols in matrix A            */
  long m;			/* number of input rows */
  long digits;			/* longs in an mp in matrix A */
  boolean hull;			/* V-representation -> H-representation  */
  boolean numincidence;		/* print number of incident facets       */
  boolean incidence;		/* print incident facets		 */
  boolean notranslation;        /* origin is in interior: do not translate */
  boolean getvolume;		/* compute the volume */
  pd_RowP translation;
} pd_Global, *pd_GlobalP;

/* Do not use autoincrement operators in arguments to these macros! */
#define rowelem(row, j) ((row)->data+(assert((unsigned)(j)<=global->d), (j))*(digits+1))
#define C(dict, j) ((dict)->cd[assert((unsigned)(j)<global->d), (j)].c)
#define CLOC(dict, j) ((dict)->cd[assert((unsigned)(j)<global->d), (j)].loc)


/* Static variables ------------------------------------------------ */

static mp mpone;		/* constant = 1 */
static long rowsize;		/* constant depending on digits and d */
#ifndef NDEBUG
static long rowCount;		/* to detect memory leaks */
#endif
static long count[4];
static boolean rayfound;	/* the output contains a ray */
static mp volume;
/* Procedures ------------------------------------------------------ */

double mytime(void)
{
#ifndef OMIT_TIMES
  struct rusage rusage;

  if (getrusage(RUSAGE_SELF, &rusage)) perror("getrusage");
  return ((double)1.0 * rusage.ru_utime.tv_sec +
	  0.000001 * rusage.ru_utime.tv_usec +
	  (double)1.0 * rusage.ru_stime.tv_sec +
	  0.000001 * rusage.ru_stime.tv_usec);
#else
  return 0.0;
#endif
} /* mytime */

pd_RowP NewRow(pd_GlobalP global)
{
  pd_RowP row;
  long j;

  row = malloc(rowsize);
#ifndef NDEBUG
  rowCount++;
#endif
  if (!row) notimpl("Not enough memory");
  row->nextrow = NULL;
  row->b = 99999;
  row->minratio = -1;
  for (j=global->d; j>=0; --j) itomp(0, rowelem(row, j));
  return row;
} /* NewRow */

#ifndef NDEBUG
void FreeRow(pd_RowP row)
{
  free(row);
  rowCount--;
} /* FreeRow */
#else
#define FreeRow(row) free(row)
#endif

void AddRowToDict(pd_GlobalP global, pd_DictP dict, pd_RowP row)
     /* Add a row to a dictionary, which must not be empty */
{
  row->nextrow = dict->firstrow;
  dict->firstrow = row;
  dict->m++;
  assert(row->b == -dict->m);
} /* AddRowToDict */


pd_RowP AddNewRowToDict(pd_GlobalP global, pd_DictP dict, int b)
     /* Add a new row to a dictionary, which must not be empty */
{
  pd_RowP row;
  
  row = NewRow(global);
  row->b = b;
  AddRowToDict(global, dict, row);
  return row;
} /* AddNewRowToDict */


void ReadInequality(pd_GlobalP global, pd_DictP dict, int i)
{
  /* Read row i of the matrix (inequality #i) from the input */
  /* numbers start with 1 */
  int j;
  mp Temp, Lcm, Gcd;
  mp *denom;
  pd_RowP row;

  row = AddNewRowToDict(global, dict, -i);
  
  /* b vector first on input line */
  denom = (mp *)malloc(sizeof(mp) * (global->d+1));
  readrat(rowelem(row,global->d),denom[global->d]);
  if (zero(denom[global->d])) {
    fprintf(stderr, "\nSyntax error: Divison by zero (inequality #%d)\n", i);
    exit(1);
  }
  if (!positive(rowelem(row, global->d)) || !positive(denom[global->d])) {
    if (global->hull) {
      fprintf(stderr, "\nVertex %d is not a vertex\n", i);
    } else
	fprintf(stderr, "\nOrigin not in the interior (inequality #%d) [not yet implemented]\n", i);
    exit(1);
  }
  copy(Lcm,denom[global->d]);
  copy(Gcd,rowelem(row,global->d));        /* Gcd of numerators */
  for (j=0;j<global->d;j++) {
    if(readrat(rowelem(row, j),denom[j])) {
      if (zero(denom[j])) {
	fprintf(stderr, "\nSyntax error: Division by zero (inequality #%d)\n", i);
	exit(1);
      }
      lcm(Lcm,denom[j]);   /* update lcm of denominators */
    }
    if( !zero(rowelem(row, j)))
      if (zero(Gcd)) copy(Gcd,rowelem(row, j));
       else {
	 copy(Temp,rowelem(row, j));
	 gcd(Gcd,Temp);   /* update gcd of numerators   */
       }
  }
  storesign(Gcd,POS); storesign(Lcm,POS);
  if( greater(Gcd,mpone) || greater(Lcm,mpone) )
    for (j=0;j<=global->d;j++) 
      {
	divint(rowelem(row, j),Gcd,Temp);  /*reduce numerators by Gcd  */
	mulint(Lcm,Temp,Temp);     /*remove denominators*/
	divint(Temp,denom[j],rowelem(row, j)); /*reduce by former denominator*/
      }
  free(denom);
} /* ReadInequality */


pd_DictP NewDict(pd_GlobalP global)
{
  /* return a dictionary containing no rows */
  pd_DictP dict;
  
  dict = malloc(sizeof(pd_Dict) + global->d * sizeof(pd_ColDesc));
  if (!dict) notimpl("Not enough memory");
  dict->m = 0;
  itomp(1L, dict->det);
  dict->detsign = POS;
  dict->firstrow = dict->lastrow = NULL;
  return dict;
} /* NewDict */


pd_DictP NewDict1(pd_GlobalP global)
{
  /* Return a dictionary containing one row (objective function),
     with cd initialized */
  pd_DictP dict;
  int j;
  
  dict = NewDict(global);
  for(j=0;j<global->d;j++) { C(dict, j)=1+j; CLOC(dict, j)=j; }    
  dict->firstrow = dict->lastrow = dict->objrow = NewRow(global);
  dict->objrow->b = 0; /* row 0 (objective function) */
  return dict;
} /* NewDict1 */


pd_DictP CopyDict(pd_GlobalP global, pd_DictP olddict)
{
  /* Make a deep copy of a dictionary including its rows.
     Complexity: nd resp md */
  pd_DictP newdict;
  pd_RowP newrow, oldrow;
  long j;

  newdict = NewDict(global);
  newdict->m = -1;
  copy(newdict->det, olddict->det);
  newdict->detsign = olddict->detsign;
  for (oldrow = olddict->firstrow; oldrow; oldrow=oldrow->nextrow) {
    newrow = NewRow(global);
    memcpy(newrow, oldrow, rowsize);
    if (newdict->lastrow) {
      newdict->lastrow->nextrow = newrow;
      newdict->lastrow = newrow;
    } else {
      newdict->lastrow = newdict->firstrow = newrow;
    }
    if (olddict->objrow == oldrow) newdict->objrow = newrow;
    newdict->m++;
  }
  assert(newdict->m == olddict->m);
  for (j=0; j<global->d; j++) newdict->cd[j] = olddict->cd[j];
  return newdict;
} /* CopyDict */


void FreeDict(pd_DictP dict)
{
  pd_RowP row, nextrow;
  
  for (row=dict->firstrow; row; row=nextrow) {
    nextrow = row->nextrow;
    FreeRow(row);
  }
  free(dict);
} /* FreeDict */


#define MAXNAME 100
void readA(pd_GlobalP *globalH, pd_DictP *rootH)
     /*       code to read in matrix of rationals             */
{
  long i, j, m, n, dec_digits=0;
  char name[MAXNAME], file[MAXNAME];
  pd_GlobalP global;
  pd_DictP dict;
  pd_RowP row;
  
  global = (pd_GlobalP) malloc(sizeof(pd_Global));
  *globalH = global;
  global->hull = FALSE;
  global->numincidence = FALSE;
  global->incidence = FALSE;
  global->notranslation = FALSE;
  global->getvolume = FALSE;
  
  file[0] = 0;
  while (1) { /* read options up to begin line */
    if (scanf("%s",name) == EOF) notimpl("Syntax error: No begin line");
    if (strcmp(name,"begin")==0) break;
    if (strcmp(name,"digits")==0) {
      if (scanf("%ld",&dec_digits) != 1)
	notimpl("Syntax error in digits line");
    } else if (!strcmp(name, "hull") || !strcmp(name,"V-representation")){
      global->hull = 1;
    } else if (file[0] == 0)
	strncpy(file, name, MAXNAME); /*first line assumed file name*/
  } /* while */

  if (dec_digits != 0)
    printf("dec digits: %ld\n", dec_digits);
  if (dec_digits <= 0){
    global->digits=DEC2DIG(DEFAULT_DIGITS);
  } else {
    global->digits=DEC2DIG(dec_digits);
  } 
  digits=global->digits;
  if (digits>MAX_DIGITS)
    notimpl("Digits is too large.\nChange MAX_DIGITS and recompile!");
  if (scanf("%ld %ld %s",&m,&n,name) != 3)
    notimpl("Syntax error in or after begin line");
  if (m <= 0) notimpl("m must be > 0");
  if (n <= 1) notimpl("d+1 must be >= 2");
  if (m < n-1 || (global->hull && m==n-1))
    notimpl("Polytope must be full-dimensional (m > d)");
  global->d=n-1;              /* d=col dimension of A */
  global->m = m;
  rowsize = sizeof(pd_Row) - sizeof(long)
            + (digits+1) * (global->d+1) * sizeof(long);
  
  printf(TITLE); printf(BIT);
  printf(VERSION);   printf(AUTHOR);
  if (file[0]) printf("\n*Input File: %s",file);

  dict = NewDict1(global);
  *rootH = dict;

  if( strcmp(name,"integer") != 0 && strcmp(name,"rational") != 0 ) {
    fprintf(stderr, "\nSyntax error: Data type must be integer or rational,"
	    " not \"%s\"\n", name);
    notimpl("");
  }

  for (i=1;i<=m;i++)	/* read inequality #i */
    ReadInequality(global, dict, i);
  
  /* read in flags */
  scanf("%s",name);
  if (strcmp(name,"end")) {
    notimpl("Syntax error: no \"end\" or wrong size of input matrix"
	    " (check m and d)!");
  }
  while ( scanf("%s",name) != EOF ) {
    if( strcmp(name,"hull")==0) {
      if (!global->hull)
	printf("\nhull option should be placed before begin line - ignored");
    }
    if( strcmp(name,"incidence")==0) {
      printf("\n*%s",name);
      global->incidence = global->numincidence = TRUE;
    }
    if( strcmp(name,"#incidence")==0) {
      printf("\n*%s",name);
      global->numincidence = TRUE;
    }
    if( strcmp(name,"notranslation")==0) {
      printf("\n*%s",name);
      global->notranslation = TRUE;
    }
    if( strcmp(name,"volume")==0) {
      if (!global->hull) {
	printf("\n*%s",name);
	global->notranslation = global->getvolume = TRUE;
      } else
	printf("\n*volume option ignored (input should be H-representation)");
    }
  }        /* end of while for reading flags */
  
  printf("\n%c-representation\nbegin\n***** %ld rational\n",
	 global->hull ? 'H' : 'V', global->d+1);

  if (!global->hull) {		/* add a 1 0 0 ... 0 row */
    /* This row has b = -m-1.
       When incidences are computed, it must not be counted. */
    row = AddNewRowToDict(global, dict, -m-1);
    for (j=0; j<global->d; j++) itomp(0, rowelem(row, j));
    itomp(1, rowelem(row, global->d));
  }
}          /* end of readA() */

void PrintPoint0(pd_GlobalP global, pd_DictP dict, pd_RowP point,
		 const char *name)
{
  /* Print the elements of a point.
     If dict is NULL, the point is in cartesian coordinates, otherwise
     relative to the basis of dict. */
  int c, j;
  pd_RowP row;
  mp newx, h;

  if (!dict||TR) {
    printf("%%Point '%s'%s: ", name, (dict?" [Var]":""));
    for (j=0; j<global->d; j++) pmp(" ", rowelem(point, j));
    pmp(" / ", rowelem(point, global->d));
    printf("\n");
  }
  if (dict) {
    printf("%%  Vars: ");
    for (c=0; c<global->d; c++) printf("%3ld", C(dict, c));
    printf("\n");
    printf("%%  Point '%s': ", name);
    for (c=1; c<=global->d; c++) { /* cartesian coordinate c */
      /* test if this coordinate is basis or cobasis */
      for (j=0; j<global->d && C(dict, j) != c; j++);
      if (j==global->d) {
	for (row = dict->firstrow; row->b!=c; row=row->nextrow);
	itomp(0, newx);
	for (j=0; j<=global->d; j++) {
	  mulint(rowelem(point, j), rowelem(row, j), h);
	  linint(newx, 1, h, 1);
	}
      } else mulint(rowelem(point, CLOC(dict, j)), dict->det, newx);
      pmp(" ", newx);
    } /* for c */
    mulint(rowelem(point, global->d), dict->det, newx);
    pmp(" / ", newx);
    printf("\n");
    printf("%%  Basis '%s': ", name);
    for (c=1; c<=global->d; c++) {
      for (row = dict->objrow; row && row->b!=c; row=row->nextrow);
      if (row) {
	pmp(" ", rowelem(row, global->d));
      } else
	  printf(" 0 ");
    }
    pmp(" / ", dict->det);
    printf("\n");
  } /* dict */
} /* PrintPoint0 */

#define PrintPoint(global, dict, point) \
        PrintPoint0(global, dict, point, #point)


void NormalizeRow(pd_GlobalP global, pd_RowP row)
{
  mp Gcd, temp;
  int j;
  
  copy(Gcd, rowelem(row, global->d));	/* initialize gcd */
  storesign(Gcd, POS);
  for (j=0; j<global->d; j++) { /* compute gcd */
    copy(temp, rowelem(row, j));
    storesign(temp, POS);
    gcd(Gcd, temp);
  }
  assert(positive(Gcd));
  if (greater(Gcd, mpone))
    for (j=0; j<=global->d; j++) { /* divide everything by gcd */
      copy(temp, rowelem(row, j));
      divint(temp, Gcd, rowelem(row, j));
      assert(zero(temp));	/* remainder */
    }
} /* NormalizeRow */

void TranslateA(pd_GlobalP global, pd_DictP v0dict)
{
  pd_RowP row;
  int j;
  mp Lcm, fact, facr, temp, temp1;

  global->translation = NewRow(global);

  for (j=0; j<=global->d; j++) itomp(0, rowelem(global->translation, j));
  if (! global->notranslation) {
    /* Compute a convex combination of all input vertices */
    for (row=v0dict->firstrow; row->b; row=row->nextrow)
      for (j=0; j<=global->d; j++)
	linint(rowelem(global->translation, j), 1,
	       rowelem(row, j), 1);

    /* Normalize the translation vector */
    NormalizeRow(global, global->translation);
    if (INF) PrintPoint(global, NULL, global->translation);

    /* Translate all input vertices */
    for (row=v0dict->firstrow; row->b; row=row->nextrow) {
      /* Lcm = lcm of both denominators */
      copy(Lcm, rowelem(global->translation, global->d));
      lcm(Lcm, rowelem(row, global->d));
      /* fact = factor for translation vector numerators */
      copy(temp, Lcm);
      divint(temp, rowelem(global->translation, global->d), fact);
      /* facr = factor for row numerators */
      copy(temp, Lcm);
      divint(temp, rowelem(row, global->d), facr);

      /* store the new denominator */
      copy(rowelem(row, global->d), Lcm);
      /* compute the new numerators */
      for (j=0; j<global->d; j++) {
	mulint(rowelem(global->translation, j), fact, temp);
	mulint(rowelem(row, j), facr, temp1);
	subint(temp1, temp);
	copy(rowelem(row, j), temp1);
      } /* for j */
      NormalizeRow(global, row);
    } /* for row */
  } else {
    /* Do not translate the input polytope. The user is responsible that
       the origin is in the interior. This lets us compute with smaller
       numerators and denominators. */
    itomp(1, rowelem(global->translation, global->d));
  }
} /* TranslateA */


void PrintDict(pd_GlobalP global, pd_DictP dict, const char *name)
{
  /* Print a dictionary for debugging. */
  long i, j, c;
  pd_RowP row;
  
  printf("\nDictionary '%s': m=%ld, d=%ld, ", name, dict->m, global->d);
  pmp("det=", dict->det);

  printf("\nPoint '%s': ", name);
  for (c=1; c<=global->d; c++) {
    for (j=0; j<global->d && C(dict, j) != c; j++);
    if (j==global->d) {
      for (row = dict->firstrow; row->b!=c; row=row->nextrow);
      pmp(" ", rowelem(row, global->d));
    } else printf(" 0");
  }
  pmp(" / ", dict->det);

  printf("\n\nColumns: variable");
  for (j=0; j<global->d; j++)
    printf("%3ld", C(dict, j));
  printf("\n         column  ");
  for (j=0; j<global->d; j++)
    printf("%3ld", CLOC(dict, j));
  printf("\n\n");
  
  printf("Rows: variable");
  row = dict->firstrow;
  for (i=0; i<=dict->m; i++) {
    assert(row!=NULL);
    printf("%3ld", row->b);
    row = row->nextrow;
  }
  assert(!row);
  printf("\n");

  row = dict->firstrow;
  for (i=0; i<=dict->m; i++) {
    assert(row!=NULL);
    if (row->b) printf("\n[%2ld]", row->b); else printf("\n( 0)");
    for (j=0; j<=global->d; j++) pmp(" ", rowelem(row, j));
    row = row->nextrow;
  }
  assert(!row);
  printf("\n");
} /* PrintDict */


void pivotone(pd_GlobalP global, pd_DictP dict,
	      pd_RowP prow, pd_RowP irow,
	      long j, long s, mp *Ars)
{
  /* pivot row: prow       pivot column: s   */
  /* element row: irow     element column: j */
  /* A(i,j)=(A(i,j)*Ars-A(i,s)*A(r,j))/det; */
  mp Nt, Ns;

  mulint(rowelem(irow, j),*Ars,Nt);
  mulint(rowelem(irow, s),rowelem(prow, j),Ns);
  subint(Nt, Ns);
  divint(Nt,dict->det,rowelem(irow, j));
} /* pivotone */

void pivot(pd_GlobalP global, pd_DictP dict, pd_RowP prow, long cob)
{
  /* Pivot operation for dictionary dict, pivot row prow and pivot column cob.
     Complexity: nd resp md */
  long s, j, d;
  mp Ars;			/* pivot element A(r,s) */
  pd_RowP irow;

  assert(prow != NULL);
  if (INF) printf("%%Pivot: %ld %ld\n", prow->b, C(dict, cob));
  s = CLOC(dict, cob);
  copy(Ars,rowelem(prow, s));
  assert(!zero(Ars));
  storesign(dict->det,sign(Ars));   /*adjust determinant to new sign*/
  
  d = global->d;
  for (irow=dict->firstrow;irow;irow=irow->nextrow) /* for all rows except r */
    if (irow != prow)
      for (j=d;j>=0;j--) /* for all columns except s */
        if ( j != s )
	  pivotone(global, dict, prow, irow, j, s, &Ars);
  if ( sign(Ars)==POS) {
    for (j=0;j<=global->d;j++)  /* no need to change sign if Ars neg*/   
      /*    pivot row: A(r,j)=-A(r,j);              */
      if (! zero(rowelem(prow, j))) changesign(rowelem(prow, j));
  } else         /* watch out for above if when removing this } ! */
      for (irow=dict->firstrow;irow;irow=irow->nextrow) /* for all rows */
        if (! zero(rowelem(irow, s))) changesign(rowelem(irow, s));
  /*  A(r,s)=det;                  */
  copy(rowelem(prow, s),dict->det);     /* restore old determinant */
  dict->detsign=sign(Ars)*dict->detsign;  /* sign of new determinant */
  copy(dict->det,Ars);
  storesign(dict->det,POS);          /* always keep positive determinant */
}                                 /* end of pivot routine */


pd_RowP ratio(pd_GlobalP global, pd_DictP dict, long col)
     /*find lex min. ratio */
     /* find min index ratio -aig/ais, ais<0 */
     /* if multiple, checks successive basis columns*/
     /* return pointer to pivot row */
     /* or return NULL if this pivot column is unbounded */
{
  long comp,degencount,ratiocol,basicindex,cindex;
  long firstime;      /*For ratio test, true on first pass,else false*/
  mp Nmin,Dmin;
  pd_RowP ird, jrd, krd, brd /*=bindex*/;

  /* search and count rows with negative coefficient (pivot) in dictionary */
  degencount=0;
  for (jrd=dict->firstrow; jrd->b; jrd=jrd->nextrow) {
    jrd->minratio=FALSE;
    if(negative(rowelem(jrd, col))) {
      jrd->minratio=TRUE;
      degencount++;
    }
  } /* end of for loop */
  if(degencount==0) return NULL; /* non-negative pivot column: unbounded */

  ratiocol=global->d;  /* column being checked, initially rhs, then others */
  brd = dict->firstrow;  /* index of next basic variable to consider */
  cindex=0;    /* index of next cobasic variable to consider */
  basicindex= -dict->m-1; /* index of basis inverse for current ratio test, except d=rhs test*/
  while (degencount > 1 ) {  /*keep going until unique min ratio found */
    if(basicindex > global->d  ) /*should never happen in lex min ratio!*/
      {PrintDict(global, dict, "Error");
      printf("Internal error: ratio test for col %ld",col);
      notimpl("failed");
      }
    if(brd->b==basicindex/*B[bindex]==basicindex*/) {
      /*in this case basis inverse is an identity column */
      /*so we do not perform a ratio test                */
      if (brd->minratio) {
	degencount--;
	brd->minratio=FALSE;
      }
      brd = brd->nextrow;
    } else {	 /* perform ratio test on rhs or column of basis inverse*/
      assert(basicindex == -dict->m-1 || basicindex == dict->cd[cindex].c);
      firstime=TRUE;
      /*get next ratio column and increment cindex*/ 
      if (basicindex != -dict->m-1) ratiocol=dict->cd[cindex++].loc;
      for (jrd=dict->firstrow; jrd->b; jrd=jrd->nextrow)
	if(jrd->minratio) {
	  /*i=LOC[B[j]];  i is the row location of the next basic variable*/
	  comp=1;       /* 1:  lhs>rhs;  0:lhs=rhs; -1: lhs<rhs */
	  if (firstime) {
	    firstime=FALSE; /*force new min ration on first time*/
	  } else {
	    if (positive(Nmin) || negative(rowelem(jrd, ratiocol))) {
	      if (!negative (Nmin) && !positive (rowelem(jrd, ratiocol))) {
		comp= -1;
	      } else comp = comprod(Nmin, rowelem(jrd, col),
				    rowelem(jrd, ratiocol),Dmin);
	    } else if (zero(Nmin) && zero(rowelem(jrd, ratiocol))) comp=0;
            if (ratiocol==global->d)
	      comp= -comp;     /* all signs reversed for rhs */
	  } /* !firstime */
	  if (comp == 1) {   /*new minimum ratio */
	    for(krd=dict->firstrow; krd!=jrd; krd=krd->nextrow)
	      krd->minratio = FALSE;
	    copy(Nmin,rowelem(jrd, ratiocol));
	    copy(Dmin,rowelem(jrd, col));
	    degencount=1;
	  } else if (comp == 0) degencount++;  /* repeated minimum */
	  else jrd->minratio=FALSE;   /* no longer minimum in this row */
	} /* end of if(minratio[j]) and for statement */
      } /* end of else statement */
      basicindex++;      /* increment column of basis inverse to check next */
      if(0/*debug*/) {
	printf(" ratiocol=%ld degencount=%ld ",ratiocol,degencount);
	printf("  Min ratios: ");
	for(ird=dict->firstrow; ird; ird=ird->nextrow) 
	  if(ird->minratio) printf(" %ld ",ird->b);
	printf("\n");
      }
    } /*end of while loop*/
  jrd = dict->firstrow;
  while ( ! jrd->minratio ) jrd=jrd->nextrow; /* find minratio to return */
  return jrd;
}                          /* end of ratio */


void update(pd_GlobalP global, pd_DictP dict, pd_RowP prow, int *j)
     /*update the B,C, LOC arrays after a pivot involving B[bas] and C[cob] */
{
  long leave,enter;
  pd_RowP *rowh;
  pd_ColDesc jcd;
  
  leave=prow->b;		/* this variable was row, will be column */
  enter=C(dict, *j);		/* this variable was column, will be row */
  C(dict, *j) = leave;
  prow->b = enter;
  /* remove prow from the list */
  for (rowh = &dict->firstrow; *rowh != prow; rowh = &((*rowh)->nextrow));
  *rowh = prow->nextrow;
  assert(prow != dict->lastrow);
  /* prow is not in the list; find place to insert */
  for (rowh = &dict->firstrow;
       *rowh && (*rowh)->b < enter; rowh = &((*rowh)->nextrow));
  prow->nextrow = *rowh;
  *rowh = prow;

  /* sort dict->cd */
  jcd = dict->cd[*j];
  while (*j && jcd.c < dict->cd[*j-1].c) { /* move left */
    dict->cd[*j] = dict->cd[*j-1];
    (*j)--;
  }
  while (*j < global->d-1 && jcd.c > dict->cd[*j+1].c) { /* move right */
    dict->cd[*j] = dict->cd[*j+1];
    (*j)++;
  }
  dict->cd[*j] = jcd;
} /* update */


void PivotValue(pd_GlobalP global, pd_DictP dict, pd_RowP prow, long cob,
		pd_RowP point)
     /* Transform a point before doing a pivot with prow and cob */
     /* This transformations modifies only one component of point and
	the denumerator */
{
  int j;
  mp newx, h, u, v, a, f;

  itomp(0, newx);
  for (j=0; j<=global->d; j++) {
    mulint(rowelem(point, j), rowelem(prow, j), h);
    linint(newx, 1, h, 1);
  }
  copy(u, newx);
  copy(v, dict->det);
  gcd(u, v);			/* u = gcd(newx, det) */
  copy(a, newx);
  divint(a, u, newx);		/* newx = newx / gcd(newx, det) */
  copy(a, dict->det);
  divint(a, u, f);		/* f = det / gcd(newx, det) */
  for (j=0; j<=global->d; j++) {
    mulint(f, rowelem(point, j), rowelem(point, j));
  }
  copy(rowelem(point, CLOC(dict, cob)), newx);
  assert(positive(rowelem(point, global->d))); /* positive denumerator */
} /* PivotValue */


pd_RowP IntersectRay(pd_GlobalP global, pd_DictP dict, pd_RowP point)
{
  /* Called from DualEdmonds with vdict.
     Intersect the ray from the current vertex through point with planes.
     Return the plane and modify point.
     Complexity: nd */
  int j;
  pd_RowP bestrd, rd;
  mp c, h, bestb, bestc, *b;

  bestrd = NULL;
  for (rd=dict->firstrow; rd->b; rd=rd->nextrow) {
    /* intersect ray with rd, compute c = sum(p[j]*a[i,j]) */
    itomp(0, c);
    for (j=0; j<global->d; j++) {
      mulint(rowelem(point, j), rowelem(rd, j), h);
      linint(c, 1, h, 1);
    }
    b = (mp*) rowelem(rd, global->d);
    /* minimize t = -b / c >= 0 */
    if (negative(c) && (!bestrd || comprod(*b, bestc, c, bestb)>0)) {
      /* c < 0; b >= 0 */
      bestrd = rd;
      copy(bestb, *b);
      copy(bestc, c);
    }
  } /* for */

  if (bestrd) { /* bounded */
    /* compute new point */
    reduce(bestb, bestc);
    storesign(bestc, POS);
    copy(rowelem(point, global->d), bestc);
    for (j=0; j<global->d; j++)
      mulint(bestb, rowelem(point, j), rowelem(point, j));
  }
  return bestrd;
} /* IntersectRay */


void invertsign(mp a)
{
  a[0] = -a[0];
} /* invertsign */


pd_RowP IntersectRay2(pd_GlobalP global, pd_DictP dict, pd_RowP point,
			  long cob)
{
  /* Called from Raindrop.
     Intersects a ray starting at any point in the relative interior with
     the polytope. cob defines the pivot column. This function returns the
     pivot row (or NULL if the ray is unbounded).
     Complexity: nd resp md */
  int j, minimize;
  pd_RowP bestrd, rd;
  mp c, h, bestb, bestc, b, pivot;

  minimize = positive(rowelem(dict->objrow, CLOC(dict, cob))); /*objective*/
  bestrd = NULL;
  for (rd=dict->firstrow; rd->b; rd=rd->nextrow) {
    /* intersect ray with rd, compute c = sum(p[j]*a[i,j]) */
    copy(pivot, rowelem(rd, CLOC(dict, cob)));
    if (minimize) invertsign(pivot);
    if (negative(pivot)) {
      itomp(0, c);
      for (j=0; j<=global->d; j++) {
	mulint(rowelem(point, j), rowelem(rd, j), h);
	linint(c, 1, h, 1);
      }
      mulint(rowelem(point, global->d), pivot, b);
      assert(!negative(c));	/* pivot row inequality is valid for point */
      assert(negative(b));
      /* minimize t = -c / b >= 0 */
      if (!bestrd || comprod(b, bestc, c, bestb)<0) {
	bestrd = rd;
	copy(bestb, b);
	copy(bestc, c);
      } /* better row */
    } /* pivot element negative */
  } /* for */
  return bestrd;
} /* IntersectRay2 */


pd_RowP NormalizedVertex(pd_GlobalP global, pd_DictP dict)
{
  /* Called by AddFacetToDict with vdict and by Phase2 with hdict.
     Complexity: d */
  int j;
  pd_RowP newrow, row;

  newrow = NewRow(global);
  
  for (j=global->d; j>=0; --j) itomp(0, rowelem(newrow, j));
  for (row = dict->objrow->nextrow; row; row=row->nextrow)
    copy(rowelem(newrow, row->b-1), rowelem(row, global->d));
#ifndef NDEBUG
  for (row = dict->firstrow; row->b; row=row->nextrow)
    assert(!negative(rowelem(row, global->d)));
#endif
  assert(positive(dict->det));
  copy(rowelem(newrow, global->d), dict->det);

  NormalizeRow(global, newrow);
  
  return newrow;
} /* NormalizedVertex */

pd_RowP Neighbor(pd_GlobalP global, pd_DictP hdict, int nr,
		 int *entering, int *exiting)
{
  /* Called by Phase3. Compute the normalized coordinates of neighbor nr
     of a vertex without doing the full pivot operation. Return NULL
     if the neighbor is equal to the current (degenerate) vertex.
     Complexity: m (for selecting the pivot row) */
  int j, col;
  pd_RowP neighbor, prow, irow;
  mp Ars, det, Nt, Ns;
  mp Nmin,Dmin;

  /*PrintDict(global, hdict, "Neighbor");*/
  col = CLOC(hdict, nr);
  *entering = C(hdict, nr);

  /* Select pivot row. If the new vertex is degenerate, it doesn't matter
     which row is chosen. */
  /* Exit if Nmin = 0 (current vertex is degenerate) */
  prow = NULL;
  for (irow=hdict->firstrow; irow->b; irow=irow->nextrow) {
    if (negative(rowelem(irow, col))) {
      if (!prow ||
	  comprod(rowelem(irow, global->d), Dmin,
		  Nmin, rowelem(irow, col)) == 1) { /* new minimum ratio */
	prow = irow;
	copy(Nmin,rowelem(irow, global->d)); /* always nonnegative */
	if (zero(Nmin)) return NULL;
	copy(Dmin,rowelem(irow, col)); /* always negative */
      } /* new minimum ratio */
    } /* negative */
  } /* for irow */
  assert(prow != NULL); /* non-negative pivot column: unbounded */
  *exiting = prow->b;
  if (INF) printf("%%Neighbor entering %d, exiting %d\n", *entering, *exiting);

  /* Allocate and initialize neighbor */
  neighbor = NewRow(global);
  for (j=global->d; j>=0; --j) itomp(0, rowelem(neighbor, j));

  /* Compute coordinates (part of the pivot operation) */
  copy(Ars,rowelem(prow, col));
  assert(!zero(Ars));
  copy(det, hdict->det);
  storesign(det, sign(Ars));
  for (irow = hdict->objrow->nextrow; irow; irow=irow->nextrow) {
    mulint(rowelem(irow, global->d),Ars,Nt);
    mulint(rowelem(irow, col),rowelem(prow, global->d),Ns);
    subint(Nt, Ns);
    divint(Nt,det,rowelem(neighbor, irow->b-1));
  }
  copy(rowelem(neighbor, global->d), Ars);
  storesign(rowelem(neighbor, global->d), POS);

  NormalizeRow(global, neighbor);
  
  return neighbor;
} /* Neighbor */

int PointInDict(pd_GlobalP global, pd_DictP dict, pd_RowP vertex)
{
  /* Called by BoundingBox.AddFacetToDict with h0dict,
     and by Phase2, Phase3 with v0dict.
     Always called with a row returned by NormalizedVertex. Test if this
     row is an element of dict.
     Precondition: the rows of dict are normalized.
     Complexity: nd resp md. A suitable data structure could reduce the
     complexity. */
  int j;
  pd_RowP row;

  for (row = dict->firstrow; row->b; row=row->nextrow) {
    for (j=0;
	 j<=global->d && !greater(rowelem(row, j), rowelem(vertex, j)) &&
	   !greater(rowelem(vertex, j), rowelem(row, j));
	 j++);
    if (j > global->d) return TRUE;
  }
  return FALSE;
} /* PointInDict */


void PrintRow(pd_GlobalP global, pd_DictP vdict, pd_RowP row, const char *name)
{
  /* Print a row of h0dict in standard format. Called by AddFacetToDict.
   The parameter vdict (if non-null) is used only to print incidences. */
  int i, j;
  int numincident;		/* number of incident facets */
  mp g, a, sum, Gcd, temp;
  pd_RowP ird;

#if 0 /* untranslated */
  printf(">>%s (", name);
  for (j=0; j<global->d; j++) {
    copy(a, rowelem(row, j));
    copy(g, rowelem(row, global->d));
    reduce(a, g);
    prat("", a, g);
  }
  printf(")");
  pmp("", rowelem(row, global->d));
  printf("\n");
#endif

#if 1 /* translated */
  /* Compute the translation */
  mulint(rowelem(row, global->d), rowelem(global->translation, global->d),
	 sum);
  for (j=0; j<global->d; j++) {
    mulint(rowelem(global->translation, j), rowelem(row, j), a);
    linint(sum, 1, a, -1);
  }
  /* sum may be zero */
  
  if (global->hull || zero(sum)) { /* output facets or rays (integers) */
    if (! global->hull) rayfound = 1; /* output warning at end */
    copy(Gcd, sum);
    storesign(Gcd, POS);
    for (j=0; j<global->d; j++) { /* compute gcd */
      mulint(rowelem(row, j), rowelem(global->translation, global->d), a);
      storesign(a, POS);
      gcd(Gcd, a);
    }
    assert(positive(Gcd));
    printf("%s", name);
    divint(sum, Gcd, temp); /* destroys sum */
    pmp("", temp);
    for (j=0; j<global->d; j++) {
      mulint(rowelem(row, j), rowelem(global->translation, global->d), a);
      divint(a, Gcd, temp);
      pmp(" ", temp);
    }
  } else { /* not hull: output vertices (rational numbers) */
    assert(positive(sum));
    printf("%s1 ", name);
    for (j=0; j<global->d; j++) {
      mulint(rowelem(row, j), rowelem(global->translation, global->d), a);
      copy(g, sum);
      reduce(a, g);
      prat("", a, g);
    }
  } /* not hull */
  
  if (vdict && global->numincidence) {	/* count / print incident vertices */
    /* What vertices are incident to the current facet?
       - The d cobasic variables of vdict are incident.
       - The basic variables of vdict with zero right-hand side are incident.
       - For vertex enumeration, the "virtual facet" 1 0 0 0 must not be
         counted. This may be basic or cobasic.

       Incidences of rays may be wrong in case of redundant hyperplanes. If
       rays are output, a warning will be generated.
       Example input (the third inequality is redundant)
       H-representation
       begin 3 3 integer
       1 1 0
       1 0 2
       1 0 1
       end
       incidence

       Example output (ray 0 1 0 should not be incident with inequality 1 0 1)
       V-representation
       begin
       ***** 3 rational
       1 -1 -1/2 : 2 1|2
       0 1 0: 3 2|2
       0 0 1: 1|1
       end
       *Number of vertices+rays: 3
     */
    numincident = global->d;	/* count incident vertices */
    if (! global->hull) numincident --; /* correct for vertex 1 0 0 ... 0 */
    for (ird=vdict->firstrow; ird->b; ird=ird->nextrow) {
      if (-ird->b > global->m)
	numincident ++; /* correct for vertex 1 0 0 ... 0 */
      else if (zero(rowelem(ird, global->d))) numincident ++;
    }
    
    if (global->incidence) {	/* print (non)incident facets */
      printf(":");
      if (1 || /*****/numincident*2 <= vdict->m) { /* print incident facets */
	i = 0;			/* index into C */
	ird = vdict->firstrow;
	while (i < global->d && ird->b)
	  if (zero(rowelem(ird, global->d))) {
	    if (C(vdict, i) < ird->b) {
	      if (-C(vdict, i) <= global->m) printf(" %ld", -C(vdict, i));
	      i++;
	    } else {
	      if (-ird->b <= global->m) printf(" %ld", -ird->b);
	      ird=ird->nextrow;
	    }
	  } else ird=ird->nextrow;
	  while (i < global->d) {
	    if (-C(vdict, i) <= global->m) printf(" %ld", -C(vdict, i));
	    i++;
	  }
	  while (ird->b) {
	    if (zero(rowelem(ird, global->d)))
	      if (-ird->b <= global->m) printf(" %ld", -ird->b);
	    ird=ird->nextrow;
	  }
      } else {	/* print nonincident facets: wrong */
	numincident = numincident - vdict->m;
	for (ird=vdict->firstrow; ird->b; ird=ird->nextrow)
	  if (! zero(rowelem(ird, global->d))) {
	    printf(" %ld", -ird->b);
	  }
      }	/* print nonincident facets */
    } /* print (non)incident facets */
    printf("|%d", numincident);
  } /* numincidence */
  printf("\n");
#endif
} /* PrintRow */


void AddFacetToDict(pd_GlobalP global, pd_DictP vdict, pd_DictP h0dict,
		    boolean duplicateCheck)
{
  /* Called by DualEdmonds (with duplicateCheck),
     and DeleteVertex, DeleteVertex3, DeleteRay (without duplicateCheck).
     Add the facet defined by vdict to h0dict and print it.
     Complexity: d (without duplicateCheck) resp md (with duplicateCheck) */
  pd_RowP newrow;

  newrow = NormalizedVertex(global, vdict);

  if (duplicateCheck)
    /* check for duplicates; not necessary when called by DeleteVertex */
    if (PointInDict(global, h0dict, newrow)) { /* duplicate */
      if (TR) PrintRow(global, vdict, newrow, "Duplicate: ");
      FreeRow(newrow);
      return;
    }

  /* insert into h0dict */
  newrow->b = - h0dict->m - 1;
  AddRowToDict(global, h0dict, newrow);

  /* print */
  PrintRow(global, vdict, newrow, "");
} /* AddFacetToDict */


#ifndef NDEBUG
void CheckFeasible(pd_GlobalP global, pd_DictP dict, pd_RowP a0)
{
  /* is a0 feasible for dict? */
  pd_RowP row;
  int j;
  mp sum, h;

  /* check columns */
  for (j=0; j<global->d; j++)
    if (C(dict, j) < 0) assert(! negative(rowelem(a0, CLOC(dict, j))));
  assert(positive(rowelem(a0, global->d))); /* denominator */

  for (row = dict->firstrow; row->b; row=row->nextrow) {
    /* check all rows */
    itomp(0, sum);
    for (j=0; j<=global->d; j++) {
      mulint(rowelem(row, j), rowelem(a0, j), h);
      linint(sum, 1, h, 1);
    }
    assert(! negative(sum));
  }
} /* CheckFeasible */


void CheckStillFeasible(pd_GlobalP global, pd_DictP v0dict, pd_DictP vdict)
{
  /* is a0 feasible for dict? */
  pd_RowP a0;

  a0 = NormalizedVertex(global, vdict);
  CheckFeasible(global, v0dict, a0);
  FreeRow(a0);
} /* CheckStillFeasible */
#else /* NDEBUG */
#define CheckFeasible(global, dict, a0)
#define CheckStillFeasible(global, v0dict, vdict)
#endif /* NDEBUG */

void ExitUnbounded(pd_GlobalP global, int pos)
{
  /*fprintf(stderr, "%d: ", pos);*/
  if (global->hull) { /* input = V-representation; output = H-representation */
    if (global->notranslation) {
      fprintf(stderr, "Origin not in the interior / not full-dimensional\n");
    } else
	fprintf(stderr, "Not full-dimensional\n");
  } else { /* input = H-representation; output = V-representation */
    if (global->notranslation) {
      fprintf(stderr, "Unbounded / not full-dimensional\n");
      if (global->getvolume)
	fprintf(stderr, "Remove the volume option and try again.\n");
    } else
	fprintf(stderr, "Not full-dimensional\n");
  }
  exit(1);
} /* ExitUnbounded */


void Raindrop(pd_GlobalP global, pd_DictP dict, pd_RowP p1, pd_RowP p0)
{
  /* Find a vertex starting from p1. */
  /* Transform p0 as necessary if it is not NULL. */
  /* Cobasis variables which are 0 will not be modified. */
  /* Called from DualEdmonds, DeleteVertex, DeleteVertex3, DeleteRay with vdict */
  /* Called from Phase2 with hdict */
  /* At the end, p1 will be zero. */
  /* Complexity: nd^2 resp md^2 */
  int j;
  pd_RowP prow;

  if (0 && TR) PrintPoint0(global, dict, p1, "start for raindrop");
  CheckFeasible(global, dict, p1);
  for (;;) {
    /* find non-zero var in p1 which influences objective function */
    for (j=0; j<global->d &&
	   ((zero(rowelem(p1, CLOC(dict, j))) && C(dict, j)<0) ||
	    zero(rowelem(dict->objrow, CLOC(dict, j))));
	 j++);

    if (j==global->d) {
      /* find non-zero var in p1; break if p1 is a vertex */
      for (j=0; j<global->d && zero(rowelem(p1, CLOC(dict, j))) &&
	     C(dict, j)<0; j++);
      if (j==global->d) break;
    }
    /* intersect the ray from p1 defined by the chosen pivot col with planes */
    prow = IntersectRay2(global, dict, p1, j);
    if (!prow) ExitUnbounded(global, 1);
    if (p0) PivotValue(global, dict, prow, j, p0); /* transform p0 */
    itomp(0, rowelem(p1, CLOC(dict, j))); /* update p1 */
    pivot(global, dict, prow, j);
    update(global, dict, prow, &j);
    if (p0) CheckFeasible(global, dict, p0);
    if (0 && TR) PrintPoint(global, dict, p1);
    if (0 && TR) PrintDict(global, dict, "Raindrop dict");
    CheckFeasible(global, dict, p1);
  } /* for (;;) */
  if (TR) PrintDict(global, dict, "Raindrop dict");
} /* Raindrop */


void DeleteRay(pd_GlobalP global, pd_DictP v0dict,
		  pd_DictP h0dict, pd_DictP hdict, int col)
{
  /* The ray defined by hdict and pivot column col is unbounded.
     Find a separating hyperplane and add it to h0dict.
     Called from FindPointedCone.
     In fact, we do not need hdict here, only the cobasis and the
     coordinates.
     Complexity: nd^2+m */
  pd_DictP vdict;
  pd_RowP a0; /* valid hyperplane going through the vertex */
  pd_RowP row;
  int j;

  assert(h0dict->m == hdict->m);
  
  /* For a0 (starting point for raindrop) use the origin */
  a0 = NewRow(global);
  for (j=0; j<global->d; j++) itomp(0, rowelem(a0, j));
  itomp(1, rowelem(a0, global->d));
  CheckFeasible(global, v0dict, a0);

  /* Define objective function for raindrop */
  /* This is the point defined by column col of hdict */
  vdict = CopyDict(global, v0dict);
  for (j=0; j<=global->d; j++) itomp(0, rowelem(vdict->objrow, j));

  copy(rowelem(vdict->objrow, C(hdict, col)-1), hdict->det);
  for (row = hdict->objrow->nextrow; row; row=row->nextrow)
    copy(rowelem(vdict->objrow, row->b-1), rowelem(row, CLOC(hdict, col)));
  if (INF)
    PrintPoint0(global, NULL, vdict->objrow,
		"Objective for dual raindrop in DeleteRay");

  Raindrop(global, vdict, a0, NULL);
  AddFacetToDict(global, vdict, h0dict, FALSE);
  FreeRow(a0);
  FreeDict(vdict);
} /* DeleteRay */


void DeleteVertex(pd_GlobalP global, pd_DictP v0dict,
		  pd_DictP h0dict, pd_DictP hdict)
{
  /* The vertex defined by hdict is not an element of v0dict.
     Find a separating hyperplane and add it to h0dict.
     Called from Phase2 (find root vertex) and Phase3 (reverse search).
     In fact, we do not need hdict here, only the cobasis and the
     coordinates.
     Complexity: nd^2+m */
  pd_DictP vdict;
  pd_RowP a0; /* valid hyperplane going through the vertex */
  pd_RowP h0row, row;
  int i, j;

  assert(h0dict->m == hdict->m);
  
  /* Compute a0 (starting point for raindrop) as sum of basis planes */
  /* hdict->cd defines which planes of h0dict are included */
  
  a0 = NewRow(global);
  for (j=0; j<=global->d; j++) itomp(0, rowelem(a0, j));

  i=0;
  for (h0row=h0dict->firstrow; h0row && i<global->d; h0row=h0row->nextrow)
    if (h0row->b == C(hdict, i)) {
      /* Add this row of h0dict to a0 */
      i++;
      CheckFeasible(global, v0dict, h0row);
      for (j=0; j<=global->d; j++)
	linint(rowelem(a0, j), 1, rowelem(h0row, j), 1);
    }
  
  if (TR)
    PrintPoint0(global, NULL, a0, "Starting inequality for dual raindrop");
  CheckFeasible(global, v0dict, a0);

  /* Define objective function for raindrop */
  /* This is the point defined by hdict */
  vdict = CopyDict(global, v0dict);
  for (row = hdict->objrow->nextrow; row; row=row->nextrow)
    copy(rowelem(vdict->objrow, row->b-1), rowelem(row, global->d));
  itomp(0, rowelem(vdict->objrow, global->d));

  Raindrop(global, vdict, a0, NULL);
  AddFacetToDict(global, vdict, h0dict, FALSE);
  FreeRow(a0);
  FreeDict(vdict);
} /* DeleteVertex */


void DeleteVertex3(pd_GlobalP global, pd_DictP v0dict,
		   pd_DictP h0dict, pd_ColDesc *oldcd, pd_RowP vertex,
		   int entering, int exiting)
{
  /* This procedure is a variant of DeleteVertex called by Phase3 (reverse
     search). The parameter lists are different; the result is the same.
     The vertex defined by hdict is not an element of v0dict.
     Find a separating hyperplane and add it to h0dict.
     Complexity: nd^2+m  */
  pd_DictP vdict;
  pd_RowP a0; /* valid hyperplane going through the vertex */
  pd_RowP h0row;
  int i, j;

  /* Compute a0 (starting point for raindrop) as sum of cobasis planes */
  /* hdict->cd defines which planes of h0dict are included */
  
  a0 = NewRow(global);
  for (j=0; j<=global->d; j++) itomp(0, rowelem(a0, j));

  i=0;
  for (h0row=h0dict->firstrow; h0row; h0row=h0row->nextrow)
    if ((i<global->d && h0row->b == oldcd[i].c) || h0row->b == exiting) {
      /* Add this row of h0dict to a0 */
      if (h0row->b != exiting) i++;
      if (h0row->b != entering) {
	for (j=0; j<=global->d; j++)
	  linint(rowelem(a0, j), 1, rowelem(h0row, j), 1);
      }
    }
  
  if (INF)
    PrintPoint0(global, NULL, a0, "Starting inequality for dual raindrop");
  CheckFeasible(global, v0dict, a0);

  /* Define objective function for raindrop = vertex */
  vdict = CopyDict(global, v0dict);
  for (j=0; j<global->d; j++)
    copy(rowelem(vdict->objrow, j), rowelem(vertex, j));
  itomp(0, rowelem(vdict->objrow, global->d));

  Raindrop(global, vdict, a0, NULL);
  AddFacetToDict(global, vdict, h0dict, FALSE);
  FreeRow(a0);
  FreeDict(vdict);
} /* DeleteVertex3 */

void AddFacetToHdict(pd_GlobalP global, pd_DictP h0dict, pd_DictP hdict)
{
  pd_RowP newrow, h0row, row;
  int k;
  mp h;

  /* Add to hdict the facet of h0dict which has been added last. */
  newrow = AddNewRowToDict(global, hdict, -hdict->m-1);
  h0row = h0dict->firstrow;
  for (k=0;k<global->d;k++)
    if (C(hdict, k) < 0) {
      itomp(0, rowelem(newrow, CLOC(hdict, k)));
    } else {
      assert(CLOC(h0dict, k) == k);
      mulint(rowelem(h0row, k), hdict->det, rowelem(newrow, CLOC(hdict, k)));
    }
  mulint(rowelem(h0row, global->d), hdict->det,
	 rowelem(newrow, global->d));
  for (row = hdict->objrow->nextrow; row; row=row->nextrow) {
    assert(row->b > 0 && row->b <= global->d);
    assert(C(h0dict, row->b-1) == row->b);
    assert(CLOC(h0dict, row->b-1) == row->b-1);
    for (k=0;k<=global->d;k++) {
      mulint(rowelem(h0row, row->b-1), rowelem(row, k), h);
      linint(rowelem(newrow, k), 1, h, 1);
    }
  }
  /* !!!!!! Divide whole row by det??? */
  assert(h0dict->m == hdict->m);
} /* AddFacetToHdict */

void FindPointedCone(pd_GlobalP global, pd_DictP v0dict, pd_DictP h0dict)
{
  pd_DictP hdict;
  int j;
  pd_RowP prow, p1;

  /* v0dict contains the input vertices and will not be changed.
     h0dict is empty and will contain the facets of a pointed cone.

     FindPointedCone is an extended raindrop algorithm. When the raindrop
     does not fall onto a facet, DeleteRay is called and an additional facet
     is generated.
     */

  /* Complexity: nd^3 */

  /* Start with an empty hdict and p1 in the origin */
  hdict = NewDict1(global);
  p1 = NewRow(global);
  for (j=0; j<global->d; j++) itomp(0, rowelem(p1, j));
  itomp(1, rowelem(p1, global->d));
  CheckFeasible(global, hdict, p1);
  
  for (;;) {
    /* find non-zero var in p1 which influences objective function */
    for (j=0; j<global->d &&
	   ((zero(rowelem(p1, CLOC(hdict, j))) && C(hdict, j)<0) ||
	    zero(rowelem(hdict->objrow, CLOC(hdict, j))));
	 j++);

    if (j==global->d) {
      /* find non-zero var in p1; break if p1 is a vertex */
      for (j=0; j<global->d && zero(rowelem(p1, CLOC(hdict, j))) &&
	     C(hdict, j)<0; j++);
      if (j==global->d) break;
    }

    /* intersect the ray from p1 defined by the chosen pivot col with planes */
    prow = IntersectRay2(global, hdict, p1, j);
    assert(! prow);
    if (!prow) {
      /* This ray is not bounded. Find a new facet by DeleteRay. */
      DeleteRay(global, v0dict, h0dict, hdict, j);
      if (INF) PrintDict(global, h0dict, "h0dict after DeleteRay");
      AddFacetToHdict(global, h0dict, hdict);
      prow = hdict->firstrow;
    }
    /*if (p0) PivotValue(global, hdict, prow, j, p0);*/ /* transform p0 */
    itomp(0, rowelem(p1, CLOC(hdict, j))); /* update p1 */
    if (INF) PrintDict(global, hdict, "hdict after DeleteRay");
    pivot(global, hdict, prow, j);
    update(global, hdict, prow, &j);
    /*if (p0) CheckFeasible(global, hdict, p0);*/
    if (INF) PrintPoint(global, hdict, p1);
    if (INF) PrintDict(global, hdict, "FindPointedCone hdict");
    CheckFeasible(global, hdict, p1);
  } /* for (;;) */

  assert(h0dict->m == global->d);
  if (INF) printf("Pointed cone found\n");
  FreeRow(p1);
  FreeDict(hdict);
} /* FindPointedCone */

void DualEdmonds(pd_GlobalP global, pd_DictP vdict, pd_DictP h0dict,
		 pd_DictP v0dict, pd_RowP p0)
{
  /* Complexity: nd^3. Parameter v0dict is only for debugging. */
  int j;
  pd_RowP p1, prow;

  p1 = NewRow(global);

  /* Initially, the current vertex is the origin */
  /* do Edmonds loop */
  for (;;) {
    CheckFeasible(global, vdict, p0);
    /* find non-zero var in p0; break if this vertex is equal to p0 */
    for (j=0; j<global->d && zero(rowelem(p0, CLOC(vdict, j))); j++);
    if (j==global->d) break;
      
    /* intersect the ray from the current vertex through p0 with planes */
    /* and assign the intersection to p0 */
    prow = IntersectRay(global, vdict, p0);
    if (!prow) ExitUnbounded(global, 2);

    /* find pivot column (non-zero var in p0 and non-zero pivot) */
    for (j=0;
	 j<global->d &&
	   (zero(rowelem(p0, CLOC(vdict, j))) ||
	    zero(rowelem(prow, CLOC(vdict, j)))); j++);
    assert(j < global->d);
    
    PivotValue(global, vdict, prow, j, p0);
    pivot(global, vdict, prow, j);
    update(global, vdict, prow, &j);
    if (TR) PrintPoint(global, vdict, p0);

    /* the current basis need not be feasible, but p0 is */
    /* p1 = p0 */
    for (j=0; j<=global->d; j++) copy(rowelem(p1, j), rowelem(p0, j));

    /* Raindrop(p1): pivot to some vertex */
    Raindrop(global, vdict, p1, p0);
    CheckStillFeasible(global, v0dict, vdict);

    /* we have found a vertex; store it in h0dict and print it*/
    if (TR) PrintDict(global, vdict, "Facet of Bounding Box");
    AddFacetToDict(global, vdict, h0dict, TRUE);
    CheckFeasible(global, v0dict, h0dict->firstrow);
    if (TR) PrintDict(global, h0dict, "H0");

  } /* for (;;) */
    
  FreeRow(p0);
  FreeRow(p1);
  if (INF) printf("The cone is closed\n");
} /* DualEdmonds */

void CloseCone(pd_GlobalP global, pd_DictP v0dict, pd_DictP h0dict)
{
  /* h0dict is a pointed cone */
  /* Close the pointed cone. */
  /* Complexity: nd^3 */
  pd_RowP axis, h0row, vrow, prow;
  int j;
  pd_DictP vdict;
  mp h, num, bestnum, bestden;
  
  vdict = CopyDict(global, v0dict);

  axis = NewRow(global);	/* a hyperplane which closes the cone */
  for (j=0; j<global->d; j++) itomp(0, rowelem(axis, j));
  itomp(1, rowelem(axis, global->d));
  for (h0row=h0dict->firstrow; h0row->b; h0row=h0row->nextrow) {
    /* Add this row of h0dict to a0 */
    CheckFeasible(global, v0dict, h0row);
    for (j=0; j<global->d; j++)
      linint(rowelem(axis, j), 1, rowelem(h0row, j), 1);
  }
  if (TR) PrintPoint0(global, NULL, axis, "Closing hyperplane");
  /* find first vertex in direction of the axis; store it in prow */
  prow = NULL;
  for (vrow=vdict->firstrow; vrow->b; vrow=vrow->nextrow) {
    itomp(0, num);
    for (j=0; j<global->d; j++) {
      mulint(rowelem(vrow, j), rowelem(axis, j), h);
      linint(num, 1, h, 1);
    }
    /* comprod works because all denominators are > 0 */
    if (!prow || comprod(num, bestden, bestnum, rowelem(vrow, global->d))>0) {
      prow = vrow;
      copy(bestnum, num);
      copy(bestden, rowelem(vrow, global->d));
    }
  } /* for vrow */
  if (INF) PrintPoint0(global, NULL, prow, "Outermost point on axis");

  /* Compute starting plane for dual Edmonds (point for primal Edmonds) */
  /* and store in axis */
  for (j=0; j<global->d; j++) {
    invertsign(rowelem(axis, j));
    mulint(bestden, rowelem(axis, j), rowelem(axis, j));
  }
  copy(rowelem(axis, global->d), bestnum);
  if (!positive(bestnum)) ExitUnbounded(global, 3);
  if (INF) PrintPoint0(global, NULL, axis, "Start for Dual Edmonds");
    
  DualEdmonds(global, vdict, h0dict, v0dict, axis);
  FreeDict(vdict);
} /* CloseCone */

pd_DictP Phase2(pd_GlobalP global, pd_DictP v0dict, pd_DictP h0dict)
{
  pd_RowP p1, newvertex, *hrow, *h0row,
    cobrowstart, *cobrow, *nexth0row, objrow;
  pd_DictP hdict;
  mp mdet;
  int i, j;

  p1 = NewRow(global);

  if (INF) printf("%%Computing root for reverse search\n");
  for (;;) {
    /* Copy h0dict to hdict and initialize p1 from last input point */
    /* Any point of V is in the polytope defined by H */
    hdict = CopyDict(global, h0dict);
    memcpy(p1, v0dict->firstrow, rowsize);
  
    /* Raindrop(p1): pivot to nearest vertex */
    Raindrop(global, hdict, p1, NULL);
    if (INF) PrintDict(global, hdict, "Vertex on H (witness or root)");

    /* Break if the new vertex is an element of V */
    newvertex = NormalizedVertex(global, hdict);
    if (PointInDict(global, v0dict, newvertex)) break;
    FreeRow(newvertex);

    /* DeleteVertex */
    DeleteVertex(global, v0dict, h0dict, hdict);
    FreeDict(hdict);
  }
  
  if (INF) PrintRow(global, hdict, newvertex, "%Root of reverse search: ");
  FreeRow(newvertex);
  FreeRow(p1);

  /* Renumber variables in hdict and h0dict. This implies reordering h0dict. */
  hrow = &hdict->firstrow;
  h0row = &h0dict->firstrow;
  cobrowstart = NULL;
  cobrow = &cobrowstart;
  i = - hdict->m;
  j = -global->d;
  while ((*h0row)->b < 0) {
    nexth0row = &(*h0row)->nextrow;
    if ((*h0row)->b == (*hrow)->b) { /* basis: numbers -m..-d-1 */
      /*printf("Renumber %d -> %d\n", (*h0row)->b, i);*/
      (*h0row)->b = (*hrow)->b = i; /* renumber */
      i ++;
      hrow = &(*hrow)->nextrow;
      h0row = nexth0row;
    } else { /* cobasis: numbers -d..-1 */
      assert((*h0row)->b == C(hdict, global->d+j));
      (*h0row)->b = C(hdict, global->d+j) = j; /* renumber */
      j ++;
      *cobrow = *h0row;		/* append to list */
      cobrow = &(*cobrow)->nextrow;
      *h0row = *nexth0row;	/* remove from list */
    }
  }
  objrow = *h0row;
  assert(objrow->b == 0);
  *cobrow = objrow;
  *h0row = cobrowstart;
  if (TR) PrintDict(global, h0dict, "After renumber");

  /* Initialize objective function for reverse search */
  copy(mdet, hdict->det);
  storesign(mdet, NEG);
  for (j=0; j<global->d; j++) copy(rowelem(hdict->objrow, j), mdet);
  itomp(0, rowelem(hdict->objrow, global->d));
  return hdict;
} /* Phase2 */

void PrintVolume(pd_GlobalP global)
{
  mp temp, dfactorial;
  int i;

  itomp(1, dfactorial);
  for(i=2;i<=global->d;i++) {
    itomp(i,temp);
    mulint(temp,dfactorial,dfactorial);
  }
  printf("*Sum of det(B)=");
  pmp("",volume);
  reduce(volume,dfactorial);
  printf("\n*Volume = ");
  prat("",volume,dfactorial);
  printf("\n");
} /* PrintVolume */


void Trailer(pd_GlobalP global, long m)
{
  if (global->hull)
    printf("end\n*Number of facets: %ld\n"
	   "*Replace ***** after begin line by number of facets\n", m);
  else
    printf("end\n*Number of vertices+rays: %ld\n"
	   "*Replace ***** after begin line by number of vertices+rays\n", m);
  if (global->numincidence && rayfound)
    printf("*** Warning: incidences of rays may be wrong in case of"
	   " redundant hyperplanes ***");
  if (global->getvolume) PrintVolume(global);
  /*printf("*DeleteVertex calls in ComputeNeighbour2: %ld\n", count[0]);
  printf("*Pivots which are no reverse step: %ld\n", count[2]);*/
  printf("*Reverse Search Bases: %ld\n", count[3]+1);
  assert(count[3] == count[1]);
} /* Trailer */

boolean reverse(pd_GlobalP global, pd_DictP dict, pd_RowP *r, long s)
     /* TRUE if B[*r] C[s] is a reverse lexicographic pivot*/
{
  long i,j,col;
  pd_RowP prow;

  col=CLOC(dict, s);
  if (! negative(rowelem(dict->objrow, col))) return FALSE;
  *r = prow = ratio(global, dict, col);
  if(!*r) notimpl("Internal error: Ray found in reverse()");
  
/* check cost row after "pivot" for smaller leaving index    */
/* ie. j s.t.  A(0,j)*A(row,col) < A(0,col)*A(row,j)     */
/* note both A(row,col) and A(0,col) are negative          */

  for (i=0; i<global->d && C(dict, i) < prow->b; i++)
    if (i != s) {
      j=CLOC(dict, i);
      if( positive(rowelem(dict->objrow,j)) || negative(rowelem(prow,j)))
	/*or else sign test fails trivially*/
	if ( ( ! negative (rowelem(dict->objrow,j)) &&
	       ! positive (rowelem(prow,j)) ) ||
	     comprod(rowelem(dict->objrow,j),rowelem(prow,col),
		     rowelem(dict->objrow,col),rowelem(prow,j)) == -1 ) {
	  return(FALSE); /*+ve cost found*/
	}
    }
  return(TRUE);
} /* reverse */

void selectpivot(pd_GlobalP global, pd_DictP dict, pd_RowP *r, int *s)
     /* select pivot indices using lexicographic rule   */
     /* pivot variables are B[*r] C[*s] in locations
                   LOC[B[*r]]    LOC[C[*s]]         */
{
  long j;
  
  /*find column with positive cost coef*/
  for (j=0;
       (j < global->d) && (!positive(rowelem(dict->objrow,CLOC(dict, j))));
       j++);
  if (j < global->d ) {		/* pivot column found!*/
    *s=j;
    *r=ratio(global, dict, CLOC(dict, j)); /*find row with min index ratio */
    /* if (*r==NULL), this pivot is unbounded */
  } else {			/* the dictionary is optimal */
    *s=global->d;
    *r=NULL; 
  }
}    /* end of select pivot        */

void Phase3(pd_GlobalP global, pd_DictP v0dict, pd_DictP h0dict,
	    pd_DictP hdict)
{
  /* Phase 3 (reverse search) */
  int j, depth;
  pd_RowP prow, vertex;
  int entering, exiting;

  if (global->getvolume) linint(volume, 1, hdict->det, 1);
  j=0;
  depth = 0;
  for (;;) { /* reverse search main (outer) loop */
    assert(h0dict->m == hdict->m);
    while (j<global->d) {		/* compute neighbor in direction j */
      if (negative(rowelem(hdict->objrow, CLOC(hdict, j)))) { /* reverse */
	for (;;) { /* ComputeNeighbor2 loop */
	  assert(h0dict->m == hdict->m);
	  if (INF) printf("%%Computing neighbor %d\n", j);
	  vertex = Neighbor(global, hdict, j, &entering, &exiting);
	  if (INF && vertex) PrintPoint(global, NULL, vertex);
	  if (!vertex || PointInDict(global, v0dict, vertex)) break;
	  
	  count[0]++;
	  if (INF) printf("%%RS Point is not in dictionary\n");

	  /* Delete this vertex by finding a separating hyperplane
	     and adding it to h0dict. */
	  DeleteVertex3(global, v0dict, h0dict,
			hdict->cd, vertex, entering, exiting);
	  FreeRow(vertex);

	  /* Add the new facet to hdict. */
	  AddFacetToHdict(global, h0dict, hdict);
	  
	} /* for (;;) ComputeNeighbor2 loop */
	if (vertex) FreeRow(vertex);
	/* neighbor j of hdict is a vertex of the input*/
	if (reverse(global, hdict, &prow, j)) { /* reverse step */
	  if (INF) printf("%%Reverse step\n");
	  pivot(global, hdict, prow, j); /* reverse pivot */
	  update(global, hdict, prow, &j);
	  if (global->getvolume) linint(volume, 1, hdict->det, 1);
	  count[1]++;
	  depth++;
	  if (TR) printf("%%depth = %d\n", depth);
	  j=0;
	} else {		/* not a reverse step */
	  if (INF) printf("%%Bad pivot\n");
	  count[2]++; /* bad pivots */
	  j++;
	}
      } else j++; /* not reverse */
    } /* reverse search inner loop (while) */
    if (depth) {		/* forward step */
      count[3]++;
      depth--;
      if (INF) printf("%%forward\n");
      selectpivot(global, hdict, &prow, &j);
      assert(prow != NULL);
      pivot(global, hdict, prow, j);
      update(global, hdict, prow, &j);
      j++;
    } else			/* terminate */
	break;
    /* Test for memory leaks */
    /*assert(rowCount == 2*(hdict->m+1) + v0dict->m+2);*/
  } /* for (;;) reverse search main loop */
} /* Phase3 */


int main(void)
{
  pd_GlobalP global;
  pd_DictP v0dict, h0dict, hdict;
  double starttime, time1, time2, time3;
  
  itomp(1, mpone);
  itomp(0, volume);
  prs_mp_Init();

  setbuf(stdout, NULL);
  readA(&global, &v0dict);
  if (TR) PrintDict(global, v0dict, "Input vertices");
  TranslateA(global, v0dict);
  if (INF) PrintDict(global, v0dict, "Input after translation");
  h0dict = NewDict1(global);

  /* Find a bounded set of halfspaces; store it in h0dict. */
  starttime = mytime();
  hdict = NewDict1(global);
  FindPointedCone(global, v0dict, h0dict);
  CloseCone(global, v0dict, h0dict);
  FreeDict(hdict);
  time1 = mytime();

  /* Find a vertex of H which is an element of V. */
  /* This vertex will be the root of the reverse search. */
  /* Return the corresponding dictionary in hdict. */
  /* h0dict may grow in this operation. */
  hdict = Phase2(global, v0dict, h0dict);
  time2 = mytime();

  /* Reverse search starting from hdict. */
  /* Whenever a vertex is found which is not in v0dict, call DeleteVertex */
  /* and add the new vertex to h0dict and hdict. */
  Phase3(global, v0dict, h0dict, hdict);
  time3 = mytime();
  Trailer(global, h0dict->m);
#ifndef OMIT_TIMES
  printf("*Time: %8.2f %8.2f %8.2f\n",
	 time1-starttime, time2-time2, time3-time2);
#endif
  return 0;
} /* main */
